@@ -11,7 +11,7 @@
 #include <skalibs/bytestr.h>
 #include <skalibs/allreadwrite.h>
 #include <skalibs/buffer.h>
-#include <skalibs/strerr2.h>
+#include <skalibs/strerr.h>
 #include <skalibs/env.h>
 #include <skalibs/stralloc.h>
 #include <skalibs/djbunix.h>
@@ -26,16 +26,7 @@
 #include "defaults.h"
 #include "initctl.h"
 
-#ifdef S6_LINUX_INIT_UTMPD_PATH
-# include <utmps/config.h>
-# define USAGE "s6-linux-init-maker [ -c basedir ] [ -u log_user ] [ -G early_getty_cmd ] [ -1 ] [ -L ] [ -p initial_path ] [ -m initial_umask ] [ -t timestamp_style ] [ -d slashdev ] [ -s env_store ] [ -e initial_envvar ... ] [ -q default_grace_time ] [ -D initdefault ] [ -n | -N ] [ -f skeldir ] [ -U utmp_user ] [ -C ] [ -B ] dir"
-# define OPTION_STRING "c:u:G:1Lp:m:t:d:s:e:E:q:D:nNf:U:CBS"
-# define UTMPS_DIR "utmps"
-#else
-# define USAGE "s6-linux-init-maker [ -c basedir ] [ -u log_user ] [ -G early_getty_cmd ] [ -1 ] [ -L ] [ -p initial_path ] [ -m initial_umask ] [ -t timestamp_style ] [ -d slashdev ] [ -s env_store ] [ -e initial_envvar ... ] [ -q default_grace_time ] [ -D initdefault ] [ -n | -N ] [ -f skeldir ] [ -C ] [ -B ] dir"
-# define OPTION_STRING "c:u:G:1Lp:m:t:d:s:e:E:q:D:nNf:CBS"
-#endif
-
+#define USAGE "s6-linux-init-maker [ -V boot_verbosity ] [ -c basedir ] [ -u log_user ] [ -G early_getty_cmd ] [ -1 ] [ -L ] [ -p initial_path ] [ -m initial_umask ] [ -t timestamp_style ] [ -d slashdev ] [ -s env_store ] [ -e initial_envvar ... ] [ -q default_grace_time ] [ -D initdefault ] [ -n | -N ] [ -f skeldir ] [ -R resourcelimits ] [ -C ] [ -B ] [ -S ] dir"
 #define dieusage() strerr_dieusage(100, USAGE)
 #define dienomem() strerr_diefu1sys(111, "stralloc_catb") ;
 
@@ -52,6 +43,7 @@ static char const *skeldir = S6_LINUX_INIT_SKELDIR ;
 static unsigned int initial_umask = 0022 ;
 static unsigned int timestamp_style = 1 ;
 static unsigned int finalsleep = 3000 ;
+static unsigned int boot_verbosity = 1 ;
 static int mounttype = 1 ;
 static int console = 0 ;
 static int logouthookd = 0 ;
@@ -60,10 +52,7 @@ static int innssync = 0 ;
 static int nologger = 0 ;
 static uid_t myuid = -1 ;
 static gid_t mygid = -1 ;
-
-#ifdef S6_LINUX_INIT_UTMPD_PATH
-static char const *utmp_user = 0 ;
-#endif
+static stralloc ressa = STRALLOC_ZERO ;
 
 typedef int writetobuf_func (buffer *, char const *) ;
 typedef writetobuf_func *writetobuf_func_ref ;
@@ -141,11 +130,13 @@ static int container_exit_script (buffer *b, char const *data)
       EXECLINE_EXTBINPREFIX "fdclose 1\n"
       EXECLINE_EXTBINPREFIX "fdclose 2\n"
       EXECLINE_EXTBINPREFIX "wait { }\n"
-      EXECLINE_EXTBINPREFIX "ifelse -X { test $HALTCODE = r } { "
+      EXECLINE_EXTBINPREFIX "ifelse -X { "
+      EXECLINE_EXTBINPREFIX "eltest $HALTCODE = r } { "
       S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init-hpr -fr") < 0
    || (!innssync && buffer_puts(b, "n") < 0)
    || buffer_puts(b, " }\n"
-      EXECLINE_EXTBINPREFIX "ifelse -X { test $HALTCODE = p } { "
+      EXECLINE_EXTBINPREFIX "ifelse -X { "
+      EXECLINE_EXTBINPREFIX "eltest $HALTCODE = p } { "
       S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init-hpr -fp") < 0
    || (!innssync && buffer_puts(b, "n") < 0)
    || buffer_puts(b, " }\n"
@@ -176,8 +167,8 @@ static int s6_svscan_log_script (buffer *b, char const *data)
   }
   if (buffer_puts(b, S6_EXTBINPREFIX "s6-log -bpd3 -- ") < 0) return 0 ;
   if (console && buffer_puts(b, "1 ") < 0) return 0 ;
-  if (timestamp_style & 1 && buffer_puts(b, "t ") < 0
-   || timestamp_style & 2 && buffer_puts(b, "T ") < 0
+  if ((timestamp_style & 1 && buffer_puts(b, "t ") < 0)
+   || (timestamp_style & 2 && buffer_puts(b, "T ") < 0)
    || buffer_puts(b, S6_LINUX_INIT_TMPFS "/" UNCAUGHT_DIR "\n") < 0)
     return 0 ;
   (void)data ;
@@ -198,7 +189,7 @@ static int shutdownd_script (buffer *b, char const *data)
   size_t sabase = satmp.len ;
   char fmt[UINT_FMT] ;
   if (!put_shebang(b)
-   || buffer_puts(b, S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init-shutdownd -c ") < 0
+   || buffer_puts(b, S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init-shutdownd -d3 -c ") < 0
    || !string_quote(&satmp, robase, strlen(robase))) return 0 ;
   if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
   satmp.len = sabase ;
@@ -251,42 +242,47 @@ static int sig_script (buffer *b, char const *option)
 static inline int stage1_script (buffer *b, char const *data)
 {
   size_t sabase = satmp.len ;
-  if (!put_shebang_options(b, "-S0")
-   || buffer_puts(b, S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init -c ") < 0
-   || !string_quote(&satmp, robase, strlen(robase))) return 0 ;
-  if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
-  satmp.len = sabase ;
+  char fmt[UINT_OFMT] ;
+  if (!put_shebang_options(b, "-S0")) return 0 ;
+  if (ressa.len)
   {
-    char fmt[UINT_OFMT] ;
-    if (buffer_puts(b, " -m 00") < 0
-     || buffer_put(b, fmt, uint_ofmt(fmt, initial_umask)) < 0) return 0 ;
+    if (buffer_puts(b, S6_EXTBINPREFIX "s6-softlimit -H ") < 0
+     || buffer_put(b, ressa.s, ressa.len) < 0) goto err ;
   }
+  if (buffer_puts(b, S6_LINUX_INIT_EXTBINPREFIX "s6-linux-init -v ") < 0
+   || buffer_put(b, fmt, uint_fmt(fmt, boot_verbosity)) < 0
+   || buffer_puts(b, " -m 00") < 0
+   || buffer_put(b, fmt, uint_ofmt(fmt, initial_umask)) < 0
+   || buffer_puts(b, " -c ") < 0
+   || !string_quote(&satmp, robase, strlen(robase))
+   || buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
+  satmp.len = sabase ;
   if (initial_path)
   {
     if (buffer_puts(b, " -p ") < 0
-     || !string_quote(&satmp, initial_path, strlen(initial_path))) return 0 ;
-    if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
+     || !string_quote(&satmp, initial_path, strlen(initial_path))
+     || buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
     satmp.len = sabase ;
   }
   if (env_store)
   {
     if (buffer_puts(b, " -s ") < 0
-     || !string_quote(&satmp, env_store, strlen(env_store))) return 0 ;
-    if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
+     || !string_quote(&satmp, env_store, strlen(env_store))
+     || buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
     satmp.len = sabase ;
   }
   if (slashdev)
   {
     if (buffer_puts(b, " -d ") < 0
-     || !string_quote(&satmp, slashdev, strlen(slashdev))) return 0 ;
-    if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
+     || !string_quote(&satmp, slashdev, strlen(slashdev))
+     || buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
     satmp.len = sabase ;
   }
   if (initdefault)
   {
     if (buffer_puts(b, " -D ") < 0
-     || !string_quote(&satmp, initdefault, strlen(initdefault))) return 0 ;
-    if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
+     || !string_quote(&satmp, initdefault, strlen(initdefault))
+     || buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
     satmp.len = sabase ;
   }
   if (mounttype == 2)
@@ -316,7 +312,7 @@ static void cleanup (char const *base)
   errno = e ;
 }
 
-static void auto_dir_internal (char const *base, char const *dir, uid_t uid, gid_t gid, unsigned int mode, int strict)
+static void auto_dir_internal (char const *base, char const *dir, uid_t uid, gid_t gid, unsigned int mode, unsigned int options)
 {
   size_t clen = strlen(base) ;
   size_t dlen = strlen(dir) ;
@@ -325,9 +321,21 @@ static void auto_dir_internal (char const *base, char const *dir, uid_t uid, gid
   fn[clen] = dlen ? '/' : 0 ;
   memcpy(fn + clen + 1, dir, dlen + 1) ;
 
+  if (options & 2)
+  {
+    for (size_t i = clen + 1 ; i < clen + 1 + dlen ; i++) if (fn[i] == '/')
+    {
+      fn[i] = 0 ;
+      if (mkdir(fn, 0755) < 0 && (errno != EEXIST || (options & 1))) goto err ;
+      fn[i] = '/' ;
+    }
+  }
+
+  if (options & 4) return ;
+
   if (mkdir(fn, mode) < 0)
   {
-    if (errno != EEXIST || strict) goto err ;
+    if (errno != EEXIST || (options & 1)) goto err ;
   }
   else
   {
@@ -498,77 +506,15 @@ static void getug (char const *base, char const *s, uid_t *uid, gid_t *gid)
   }
 }
 
-#ifdef S6_LINUX_INIT_UTMPD_PATH
-
-static inline void auto_basedir (char const *base, char const *dir, uid_t uid, gid_t gid, unsigned int mode)
-{
-  size_t n = strlen(dir) ;
-  char tmp[n + 1] ;
-  for (size_t i = 0 ; i < n ; i++)
-  {
-    if ((dir[i] == '/') && i)
-    {
-      tmp[i] = 0 ;
-      auto_dir_internal(base, tmp, uid, gid, mode, 0) ;
-    }
-    tmp[i] = dir[i] ;
-  }
-}
-
-static int utmpd_script (buffer *b, char const *aux)
-{
-  size_t sabase = satmp.len ;
-  if (!put_shebang(b)
-   || buffer_puts(b,
-    EXECLINE_EXTBINPREFIX "fdmove -c 2 1\n") < 0) return 0 ;
-  if (utmp_user)
-  {
-    if (buffer_puts(b, S6_EXTBINPREFIX "s6-setuidgid ") < 0
-     || !string_quote(&satmp, utmp_user, strlen(utmp_user))) return 0 ;
-    if (buffer_put(b, satmp.s + sabase, satmp.len - sabase) < 0) goto err ;
-    satmp.len = sabase ;
-    if (buffer_put(b, "\n", 1) < 0) return 0 ;
-  }
-  if (buffer_puts(b,
-    EXECLINE_EXTBINPREFIX "cd " S6_LINUX_INIT_TMPFS "/" UTMPS_DIR "\n"
-    EXECLINE_EXTBINPREFIX "fdmove 1 3\n"
-    S6_EXTBINPREFIX "s6-ipcserver -1 -c 1000 -- " UTMPS_UTMPD_PATH "\n"
-    UTMPS_EXTBINPREFIX "utmps-utmpd\n") < 0) return 0 ;
-  (void)aux ;
-  return 1 ;
-
- err:
-  satmp.len = sabase ;
-  return 0 ;
-}
-
-static inline void make_utmps (char const *base)
-{
-  auto_dir(base, "run-image/" SCANDIR "/utmpd", 0, 0, 0755) ;
-  auto_file(base, "run-image/" SCANDIR "/utmpd/notification-fd", "3\n", 2) ;
-  auto_script(base, "run-image/" SCANDIR "/utmpd/run", &utmpd_script, 0) ;
-  {
-    uid_t uid ;
-    gid_t gid ;
-    getug(base, utmp_user, &uid, &gid) ;
-    auto_dir(base, "run-image/" UTMPS_DIR, uid, gid, 0755) ;
-    auto_basedir(base, "run-image/" S6_LINUX_INIT_UTMPD_PATH, uid, gid, 0755) ;
-  }
-}
-
-#endif
-
 static inline void make_image (char const *base)
 {
-  auto_dir(base, "run-image", 0, 0, 0755) ;
-  auto_dir(base, "run-image/" SCANDIR, 0, 0, 0755) ;
-  auto_dir(base, "run-image/" SCANDIR "/.s6-svscan", 0, 0, 0755) ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGQUIT", &put_shebang_options, 0) ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGINT", &sig_script, "-r") ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGUSR1", &sig_script, "-p") ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGUSR2", &sig_script, "-h") ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGPWR", &sig_script, "-p") ;
-  auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGWINCH", &put_shebang_options, 0) ;
+  auto_dir_internal(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan", 0, 0, 0755, 3) ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGQUIT", &put_shebang_options, 0) ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGINT", &sig_script, "-r") ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGUSR1", &sig_script, "-p") ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGUSR2", &sig_script, "-h") ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGPWR", &sig_script, "-p") ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGWINCH", &put_shebang_options, 0) ;
 
   if (!nologger)
   {
@@ -576,50 +522,47 @@ static inline void make_image (char const *base)
     gid_t gid ;
     getug(base, log_user, &uid, &gid) ;
     auto_dir(base, "run-image/" UNCAUGHT_DIR, uid, gid, 02750) ;
-    auto_dir(base, "run-image/" SCANDIR "/" LOGGER_SERVICEDIR, 0, 0, 0755) ;
-    auto_fifo(base, "run-image/" SCANDIR "/" LOGGER_SERVICEDIR "/" LOGGER_FIFO) ;
-    auto_file(base, "run-image/" SCANDIR "/" LOGGER_SERVICEDIR "/notification-fd", "3\n", 2) ;
-    auto_script(base, "run-image/" SCANDIR "/" LOGGER_SERVICEDIR "/run", &s6_svscan_log_script, 0) ;
+    auto_dir(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGGER_SERVICEDIR, 0, 0, 0755) ;
+    auto_fifo(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGGER_SERVICEDIR "/" LOGGER_FIFO) ;
+    auto_file(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGGER_SERVICEDIR "/notification-fd", "3\n", 2) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGGER_SERVICEDIR "/run", &s6_svscan_log_script, 0) ;
   }
 
-  auto_dir(base, "run-image/" SCANDIR "/" SHUTDOWND_SERVICEDIR, 0, 0, 0755) ;
-  auto_fifo(base, "run-image/" SCANDIR "/" SHUTDOWND_SERVICEDIR "/" SHUTDOWND_FIFO) ;
-  auto_script(base, "run-image/" SCANDIR "/" SHUTDOWND_SERVICEDIR "/run", &shutdownd_script, 0) ;
+  auto_dir(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" SHUTDOWND_SERVICEDIR, 0, 0, 0755) ;
+  auto_fifo(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" SHUTDOWND_SERVICEDIR "/" SHUTDOWND_FIFO) ;
+  auto_file(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" SHUTDOWND_SERVICEDIR "/notification-fd", "3\n", 2) ;
+  auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" SHUTDOWND_SERVICEDIR "/run", &shutdownd_script, 0) ;
 
   if (inns)
   {
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGTERM", &sig_script, "-h") ;
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/crash", &container_crash_script, 0) ;
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/finish", &container_exit_script, 0) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGTERM", &sig_script, "-h") ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/crash", &container_crash_script, 0) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/finish", &container_exit_script, 0) ;
     auto_dir(base, "run-image/" CONTAINER_RESULTS, 0, 0, 0755) ;
     auto_file(base, "run-image/" CONTAINER_RESULTS "/exitcode", "0\n", 2) ;
   }
   else
   {
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/SIGTERM", &put_shebang_options, 0) ;
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/crash", &death_script, "crashed") ;
-    auto_script(base, "run-image/" SCANDIR "/.s6-svscan/finish", &death_script, "exited") ;
-    auto_dir(base, "run-image/" SCANDIR "/" RUNLEVELD_SERVICEDIR, 0, 0, 0755) ;
-    auto_file(base, "run-image/" SCANDIR "/" RUNLEVELD_SERVICEDIR "/notification-fd", "3\n", 2) ;
-    auto_script(base, "run-image/" SCANDIR "/" RUNLEVELD_SERVICEDIR "/run", &runleveld_script, 0) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/SIGTERM", &put_shebang_options, 0) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/crash", &death_script, "crashed") ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/.s6-svscan/finish", &death_script, "exited") ;
+    auto_dir(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" RUNLEVELD_SERVICEDIR, 0, 0, 0755) ;
+    auto_file(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" RUNLEVELD_SERVICEDIR "/notification-fd", "3\n", 2) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" RUNLEVELD_SERVICEDIR "/run", &runleveld_script, 0) ;
   }
 
   if (logouthookd)
   {
-    auto_dir(base, "run-image/" SCANDIR "/" LOGOUTHOOKD_SERVICEDIR, 0, 0, 0755) ;
-    auto_file(base, "run-image/" SCANDIR "/" LOGOUTHOOKD_SERVICEDIR "/notification-fd", "1\n", 2) ;
-    auto_script(base, "run-image/" SCANDIR "/" LOGOUTHOOKD_SERVICEDIR "/run", &logouthookd_script, 0) ;
+    auto_dir(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGOUTHOOKD_SERVICEDIR, 0, 0, 0755) ;
+    auto_file(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGOUTHOOKD_SERVICEDIR "/notification-fd", "1\n", 2) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" LOGOUTHOOKD_SERVICEDIR "/run", &logouthookd_script, 0) ;
   }
 
   if (early_getty)
   {
-    auto_dir(base, "run-image/" SCANDIR "/" EARLYGETTY_SERVICEDIR, 0, 0, 0755) ;
-    auto_script(base, "run-image/" SCANDIR "/" EARLYGETTY_SERVICEDIR "/run", &line_script, early_getty) ;
+    auto_dir(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" EARLYGETTY_SERVICEDIR, 0, 0, 0755) ;
+    auto_script(base, "run-image/" S6_LINUX_INIT_SCANDIR "/" EARLYGETTY_SERVICEDIR "/run", &line_script, early_getty) ;
   }
-
-#ifdef S6_LINUX_INIT_UTMPD_PATH
-  if (utmp_user) make_utmps(base) ;
-#endif
 }
 
 static inline void make_scripts (char const *base)
@@ -642,6 +585,80 @@ static inline void make_bins (char const *base)
   if (!inns) auto_exec(base, "bin/telinit", "s6-linux-init-telinit") ;
 }
 
+static uint8_t rclass (char c)
+{
+  switch (c)
+  {
+    case 0 : return 0 ;
+    case 'a' :
+    case 'c' :
+    case 'd' :
+    case 'f' :
+    case 'l' :
+    case 'm' :
+    case 'o' :
+    case 'p' :
+    case 'r' :
+    case 's' :
+    case 't' : return 1 ;
+    case '=' : return 2 ;
+    case '0' :
+    case '1' :
+    case '2' :
+    case '3' :
+    case '4' :
+    case '5' :
+    case '6' :
+    case '7' :
+    case '8' :
+    case '9' : return 3 ;
+    case ',' :
+    case ';' :
+    case ' ' : return 4 ;
+    default : return 5 ;
+  }
+}
+
+static void parse_resources (stralloc *sa, char const *s)
+{
+  static uint8_t const table[4][6] =
+  {
+    { 0x04, 0x11, 0x05, 0x05, 0x00, 0x05 },
+    { 0x05, 0x05, 0x23, 0x42, 0x05, 0x05 },
+    { 0x84, 0x05, 0x05, 0x02, 0x80, 0x05 },
+    { 0x04, 0x05, 0x05, 0x05, 0x00, 0x05 }
+  } ;
+  char const *m = s ;
+  uint8_t state = 0 ;
+  char tmp[UINT64_FMT] ;
+  sa->len = 0 ;
+  while (state < 4)
+  {
+    uint8_t c = table[state][rclass(*s)] ;
+    state = c & 0x07 ;
+    if (c & 0x10)
+    {
+      tmp[0] = '-' ; tmp[1] = *s ;
+      if (!stralloc_catb(sa, tmp, 2)) dienomem() ;
+    }
+    if (c & 0x20) if (!stralloc_catb(sa, "= ", 2)) dienomem() ;
+    if (c & 0x40) m = s ;
+    if (c & 0x80)
+    {
+      uint64_t u ;
+      if (s - m >= UINT64_FMT) strerr_dief1x(100, "resource limit out of range") ;
+      memcpy(tmp, m, s - m) ;
+      tmp[s - m] = 0 ;
+      if (!uint640_scan(tmp, &u)) strerr_dief1x(100, "resource limit out of range") ;
+      tmp[s - m] = ' ' ;
+      if (!stralloc_catb(sa, tmp, s - m + 1)) dienomem() ;
+    }
+    s++ ;
+  }
+  if (state > 4) strerr_dief1x(100, "syntax error in resource limit string") ;
+  if (sa->len) if (!stralloc_catb(sa, "--\n", 3)) dienomem() ;
+}
+
 int main (int argc, char const *const *argv, char const *const *envp)
 {
   PROG = "s6-linux-init-maker" ;
@@ -649,10 +666,11 @@ int main (int argc, char const *const *argv, char const *const *envp)
     subgetopt l = SUBGETOPT_ZERO ;
     for (;;)
     {
-      int opt = subgetopt_r(argc, argv, OPTION_STRING, &l) ;
+      int opt = subgetopt_r(argc, argv, "V:c:u:G:1Lp:m:t:d:s:e:E:q:D:nNf:R:CBS", &l) ;
       if (opt == -1) break ;
       switch (opt)
       {
+        case 'V' : if (!uint0_scan(l.arg, &boot_verbosity)) dieusage() ; break ;
         case 'c' : robase = l.arg ; break ;
         case 'u' : log_user = l.arg ; break ;
         case 'G' : early_getty = l.arg ; break ;
@@ -669,9 +687,7 @@ int main (int argc, char const *const *argv, char const *const *envp)
         case 'n' : mounttype = 2 ; break ;
         case 'N' : mounttype = 0 ; break ;
         case 'f' : skeldir = l.arg ; break ;
-#ifdef S6_LINUX_INIT_UTMPD_PATH
-        case 'U' : utmp_user = l.arg ; break ;
-#endif
+        case 'R' : parse_resources(&ressa, l.arg) ; break ;
         case 'C' : inns = 1 ; break ;
         case 'B' : nologger = 1 ; break ;
         case 'S' : innssync = 1 ; break ;
